% This code tests the performance of SS3COMP-C on the EYaleB face database.
% The code generates results in Table 1 of the paper
% "Stochastic Sparse Subspace Clustering", CVPR 20.

% In this code, we apply SSC-OMP to the face images of 38 subjects in the Extended Yale B database. Each
% subject has 64 images under different illumination conditions. 

% Instructions for running the code:
% - Download code for computing clustering accuracy. Go to
% http://www.cad.zju.edu.cn/home/dengcai/Data/Clustering.html and download
% bestMap.m and Hungarian.m. Alternatively, you can use your own function
% by redefining the function evalAccuracy.m
% - vl_feat toolbox. Download and install the vl_feat toolbox from
%   http://www.vlfeat.org/. The code uses the functions vl_kdtreebuild, 
%   vl_kdtreequery and vl_kmeans from this toolbox. Alternatively, one can
%   also use these functionalities provided with Matlab. 
% - Run. 

%% Setting
clear
nExperiment = 10;
nCluster = 38; % number of subjects.
buildRepresentation = @(data,dic,ind_1,lambda,C_final) S3COMP_C(data,dic,ind_1,lambda,C_final,5, 1e-6);
% second parameter is sparsity
normalizeColumn = @(data) cnormalize_inplace(data);
%% Load data
addpath('EYaleB')
addpath('toolbox')
addpath('vlfeat-0.9.21')
run('vlfeat-0.9.21/toolbox/vl_setup')


load ExtendedYaleB.mat EYALEB_DATA EYALEB_LABEL
N_subject = length(unique(EYALEB_LABEL));
subjectIdx = randperm(N_subject, nCluster); % select #nCluster subjects
datapointIdx = find(ismember(EYALEB_LABEL, subjectIdx));
X = double(EYALEB_DATA(:, datapointIdx)); % data
s = EYALEB_LABEL(datapointIdx); % label
N = length(s);
% clustering
% normalization
X = normalizeColumn(X);
%% Test begin
% timearray1=zeros(1,T+1);
% timearray2=zeros(1,nExperiment);
delta=0.1;  
T=15;
lambda=0.01;

maxiter=3;
results = zeros(nExperiment, 5); %results
results_conn=zeros(nExperiment,nCluster+4);
% CoreNum=10;
% parpool ('local',CoreNum);
for iExperiment = 1:nExperiment
    C=cell(1,T+1);
    count=zeros(1,N);
    idxcell=cell(1,T+1);
    for t=1:T
        a=rand(1,N);
        a=a>delta;
        idx=find(a>0);
        count(idx)=count(idx)+1;
        idxcell{t}=idx;
    end
    if all(count)
        T_all=T;
    else
        id1=find(count<1);
        a=rand(1,N);
        a=a>delta;
        idx=find(a>0);
        idx=union(idx,id1);
        count(idx)=count(idx)+1;
        idxcell{T+1}=idx;
        T_all=T+1;
    end
    C_final=sparse(N,N);
    time1=0;
    time2=0;
    for iter=1:maxiter
        for t=1:T_all
            tic
            C_final_t=C_final(idxcell{t},:);
            C{t}=buildRepresentation(X,X(:,idxcell{t})*(1/(1-delta)),idxcell{t},lambda,C_final_t);
            time_tmp=toc;
            time2=max(time_tmp,time2);
        end
        tic
        R=sparse(N,N);
        for t=1:T_all
            R=R+C{t};
        end
        C_final=R/T_all;
        time_tmp=toc;
        time1=time1+time_tmp+time2;
    end
    tic
    C_final(1:N+1:end) = 0;
    A = abs(C_final) + abs(C_final)';
    groups = SpectralClustering(A, nCluster, 'Eig_Solver', 'eigs','Kmeans_Solver', 'vl_kmeans');  
    time_tmp=toc;
    time1=time1+time_tmp;
    % Evaluation
    perc = evalSSR_perc( C_final, s );
    ssr = evalSSR_error( C_final, s ); 
    [conn,conn_min,conn_max,conn_median,conn_mean] = evalConn_all( A, s);
%     conn = evalConn_mean( A, s);
    accr  = evalAccuracy(s, groups);
    results_conn(iExperiment,:)=[conn,conn_min,conn_max,conn_median,conn_mean];
    % output
    dataformat = '%d-th experiment: perc = %f, ssr = %f, conn_mean = %f, accr = %f\n';
    dataValue = [iExperiment, perc, ssr, conn_mean, accr];
    fprintf(dataformat, dataValue);
    % record
    results(iExperiment, :) = dataValue;
end
% output
dataValue = mean(results, 1);
dataValue_conn=mean(results_conn,1);
fprintf('\nAverage: perc = %f, ssr = %f, conn_mean = %f, accr = %f\n', dataValue(2:end));