% This code tests the performance of S3COMP-C on the MNIST digit database.
% The code generates results in Table 5 of the paper
% "Stochastic Sparse Subspace Clustering", CVPR 20.

% Dependencies:
% - Database. Download the database from the link
%   http://benchmark.ini.rub.de/Dataset/GTSRB_Final_Training_HOG.zip
%   and put the folder "Final_Training" under the folder "GTSRB". Then, run
%   the following lines of code
% for ii = 0:42
%     [GTSRB_DATA, GTSRB_LABEL, GTSRB_NAME] = load_GTSRB(ii, 0:29, 'HOG/HOG_01');
%     eval(['save Final_Training/HOG/HOG_01/' sprintf('%05d', ii) '.mat GTSRB_DATA GTSRB_LABEL GTSRB_NAME'])
% end
%   this will generate 43 .mat files under Final_Training\HOG\HOG_01 and 
%   each file contains HOG features for all images in one of the 43 
%   categories. This allows us to load the data more easily by using the
%   function load_GTSRB_from_mat (see the code below)
% - vl_feat toolbox. Download and install the vl_feat toolbox from
%   http://www.vlfeat.org/. The code uses the functions vl_kdtreebuild, 
%   vl_kdtreequery and vl_kmeans from this toolbox. Alternatively, one can
%   also use these functionalities provided with Matlab. 

% Acknowledgement: the files bestMap.m and Hungarian.m are downloaded from
% http://www.cad.zju.edu.cn/home/dengcai/Data/Clustering.html
% - Run. 
%% Setting
clear
buildRepresentation = @(data,dic,ind_1,lambda,C_final) S3COMP_C(data,dic,ind_1,lambda,C_final,3, 1e-6);
%% Load data
addpath('GTSRB');
addpath('toolbox')
addpath('vlfeat-0.9.21')
run('vlfeat-0.9.21/toolbox/vl_setup')

categories = [12:17, 32, 34:40]; % list of image categories
[data, label, ~] = load_GTSRB_from_mat(categories, 'HOG/HOG_01');
N = length(label);
nCluster = length(categories);
nExperiment = 10;

%% Test begin
delta=0.8;
T=15;
lambda=0.8;
CoreNum=10;
parpool ('local',CoreNum);
maxiter=linspace(1,10,10);
results = zeros(length(maxiter),5);
for ii=1:length(maxiter)
    maxiter_ii=maxiter(ii);
    results_1=zeros(nExperiment, 5);
    parfor iExperiment = 1:nExperiment
        zz = RandStream('mt19937ar','Seed',iExperiment);
        RandStream.setGlobalStream(zz);
        mask = randperm(N);
        X = data(:, mask);
        s = label(:, mask);
        R = sparse(N, N);
        X = dimReduction(X, 500); % dimension reduction by PCA 
        X = bsxfun(@minus, X, mean(X, 2)); % mean subtraction
        X = cnormalize_inplace(X);
        
        C=cell(1,T+1);
        count=zeros(1,N);
        idxcell=cell(1,T+1);
        for t=1:T
            a=rand(1,N);
            a=a>delta;
            idx=find(a>0);
            count(idx)=count(idx)+1;
            idxcell{t}=idx;
        end
        if all(count)
            T_all=T;
        else
            id1=find(count<1);
            a=rand(1,N);
            a=a>delta;
            idx=find(a>0);
            idx=union(idx,id1);
            count(idx)=count(idx)+1;
            idxcell{T+1}=idx;
            T_all=T+1;
        end
        C_final=sparse(N,N);
        time1=0;
        time2=0;
        for iter=1:maxiter_ii
            for t=1:T_all
                tic
                C_final_t=C_final(idxcell{t},:);
                C{t}=buildRepresentation(X,X(:,idxcell{t})*(1/(1-delta)),idxcell{t},lambda,C_final_t);
                time_tmp=toc;
                time2=max(time_tmp,time2)
            end
            tic
            R=sparse(N,N);
            for t=1:T_all
                R=R+C{t};
            end
            C_final=R/T_all;
            time_tmp=toc;
            time1=time1+time_tmp+time2;
        end
        tic
        C_final(1:N+1:end) = 0;
        A = abs(C_final) + abs(C_final)';
        groups = SpectralClustering(A, nCluster, 'Eig_Solver', 'eigs','Kmeans_Solver', 'vl_kmeans');  
        time_tmp=toc;
        time1=time1+time_tmp;
                % Evaluation
        [perc, vec] = evalSSR_perc( C_final, s );
        ssr = evalSSR_error( C_final, s );   
        conn = evalConn( A, s);
        accr  = evalAccuracy(s, groups);

        dataformat = '%d-th experiment: perc = %f, ssr = %f, conn = %f, accr = %f,time=%f\n';
        dataValue = [iExperiment, perc, ssr, conn, accr,time1];
        fprintf(dataformat, dataValue);
        results_1(iExperiment, :) = dataValue(2:end);  
    end
    tmp_re=mean(results_1, 1);
    results(ii,:)=tmp_re;
    dataformat = 'delta=%f,T=%f,lambda=%f,maxiter=%d: perc = %f, ssr = %f, conn = %f, accr = %f, time=%f\n';
    dataValue = [delta,T,lambda,maxiter(ii),tmp_re];
    fprintf(dataformat, dataValue);
    fprintf('\n');
end
delete(gcp);