% This code tests the performance of S3COMP-C on the synthetic database.
% The code generates results in Figure 1 of the paper
% "Stochastic Sparse Subspace Clustering", CVPR 20.

% The data are drawn from 5 subspaces of dimension 6 in ambient dimension
% 9. Each subspace contains the same number of points and the overall 
% number of points is varied from 150 to 10^5.

% Instructions for running the code:
% - Download code for computing clustering accuracy. Go to
% http://www.cad.zju.edu.cn/home/dengcai/Data/Clustering.html and download
% bestMap.m and Hungarian.m. Alternatively, you can use your own function
% by redefining the function evalAccuracy.m
% - Run. 

%% Setting
% data
clear
D = 9; % ambient dimension
n = 5; % number of subspaces
di = 6; % dimension of subspaces
sigma = 0.00; % noise level

nExperiment = 10;
buildRepresentation = @(data,dic,ind_1,lambda,C_final) S3COMP_C(data,dic,ind_1,lambda,C_final,6, 1e-6);
% second parameter is sparsity
genLabel = @(affinity, nCluster) SpectralClustering(affinity, nCluster, 'Eig_Solver', 'eigs');
% p=logspace(log10(0.05), log10(1), 20);
%% Test begin
delta=0.6;
T=15;
lambda=1;
CoreNum=10;
parpool ('local',CoreNum);
Ni = 3396;  % number of points per subspace
maxiter=linspace(1,10,10);
results = zeros(length(maxiter),5);
for ii=1:length(maxiter)
    maxiter_ii=maxiter(ii);
    results_1=zeros(nExperiment, 5);
    if length(Ni) == 1
        Ni = repmat(Ni, 1, n);
    end
    parfor iExperiment = 1:nExperiment
        % Generate subspaces
        %rng(iExperiment);
        zz = RandStream('mt19937ar','Seed',iExperiment);
        RandStream.setGlobalStream(zz);

        [X, s] = genSubspace(D, n, Ni, di, sigma);
        N = sum(Ni);
        C=cell(1,T+1);
        count=zeros(1,N);
        idxcell=cell(1,T+1);
        for t=1:T
            a=rand(1,N);
            a=a>delta;
            idx=find(a>0);
            count(idx)=count(idx)+1;
            idxcell{t}=idx;
        end
        if all(count)
            T_all=T;
        else
            id1=find(count<1);
            a=rand(1,N);
            a=a>delta;
            idx=find(a>0);
            idx=union(idx,id1);
            count(idx)=count(idx)+1;
            idxcell{T+1}=idx;
            T_all=T+1;
        end 
        C_final=sparse(N,N);
        time1=0;
        time2=0;
        for iter=1:maxiter_ii
            for t=1:T_all
                tic
                C_final_t=C_final(idxcell{t},:);
                C{t}=buildRepresentation(X,X(:,idxcell{t})*(1/(1-delta)),idxcell{t},lambda,C_final_t);
                time_tmp=toc;
                time2=max(time_tmp,time2)
            end
            tic
            R=sparse(N,N);
            for t=1:T_all
                R=R+C{t};
            end
            C_final=R/T_all;
            time_tmp=toc;
            time1=time1+time_tmp+time2;
        end
        tic
        C_final(1:N+1:end) = 0;
        A = abs(C_final) + abs(C_final)';
        groups = genLabel(A, n);  
        time_tmp=toc;
        time1=time1+time_tmp;

        % Evaluation
        [perc, vec] = evalSSR_perc( C_final, s );
        ssr = evalSSR_error( C_final, s );   
        conn = evalConn( A, s);
        accr  = evalAccuracy(s, groups);

        dataformat = '%d-th experiment: perc = %f, ssr = %f, conn = %f, accr = %f,time=%f\n';
        dataValue = [iExperiment, perc, ssr, conn, accr,time1];
        fprintf(dataformat, dataValue);
        results_1(iExperiment, :) = dataValue(2:end);  
    end
    tmp_re=mean(results_1, 1);
    results(ii,:)=tmp_re;
    dataformat = 'Ni = %d,delta=%f,T=%f,lambda=%f,maxiter=%d: perc = %f, ssr = %f, conn = %f, accr = %f, time=%f\n';
    dataValue = [Ni(1),delta,T,lambda,maxiter(ii),tmp_re];
    fprintf(dataformat, dataValue);
    fprintf('\n');
end
delete(gcp);